#!/usr/bin/python
#-----------------------------------------------------------------------
# File    : counts.py
# Contents: count patterns by size and support in independent data
# Author  : Christian Borgelt
# History : 2012.06.28 file created
#           2013.01.10 progress reporting extended
#           2013.01.11 ported to psts.py and fim library
#           2013.01.12 hole filling in pattern counters added
#           2013.01.25 conditional import from pyfim added
#           2013.02.09 made compatible with Python 3 (print, range)
#           2013.05.09 filling holes in pattern spectrum improved
#           2013.12.06 filling holes replaced by detection border
#           2014.05.27 modules placed in sibling directories
#-----------------------------------------------------------------------
from sys     import argv, path
from os      import makedirs
from os.path import join
from time    import strftime
from random  import seed
UTILDIR  = join('..', 'util')
if UTILDIR  not in path: path.insert(0, UTILDIR)
from id2par import run
NEURODIR = join('..', 'neuro')
if NEURODIR not in path: path.insert(0, NEURODIR)
from psts   import genpsts, psts2tras
try:
    from fim   import fim
except ImportError:
    FIMDIR = join('..', 'fim')
    if FIMDIR not in path: path.insert(0, FIMDIR)
    from pyfim import fim
    print('Attention: Could not import from PyFIM library!')
    print('           Using Python replacement (very slow)!')

#-----------------------------------------------------------------------

def counts (ncnt=100, period=3, rate=20, width=0.003,
            target='c', runs=1000, verbose=100, srng=1):
    if srng == 0: srng = 1      # init. seed and get data file name
    name = 'n%d_p%g_r%g_w%g_%c' % (ncnt,period,rate,width*1000,target)
    psp  = dict()               # create a pattern spectrum
    for i in range(runs):       # repeat the experiments multiple times
        if verbose > 0 and i % verbose == 0:
            print(strftime('%Y.%m.%d %H:%M:%S counts ')+name+' '+str(i))
        seed(srng)              # seed random number generator
        psts = genpsts(ncnt, period, rate)
        pcs  = fim(psts2tras(psts, width, period),
                   target=target, supp=-2, zmin=2, report='#')
        for s in pcs:           # generate and mine spike trains
            if s in psp: psp[s] += pcs[s]
            else:        psp[s]  = pcs[s]
        # slower alternative to a pattern spectrum
        # pats = fim(psts2tras(psts, width, period),
        #            target=target, supp=-2, zmin=2)
        # for p,r in pats:        # generate and mine spike trains
        #     s = (len(p),r)      # and traverse the found patterns
        #     if s in psp: psp[s] += 1
        #     else:        psp[s]  = 1
        srng += 2               # update the pattern counters
    if verbose > 0:             # print termination log message
        print(strftime('%Y.%m.%d %H:%M:%S counts ')
                      +name+' '+str(runs)+' done.')
    try: makedirs('data')       # ensure that a directory exists
    except OSError: pass        # for the result files
    n = 1.0/runs if runs > 0 else 1.0
    with open(join('data','pc_'+name+'.txt'), 'w') as out:
        for x in sorted([(s[0], s[1], psp[s]*n) for s in psp]):
            out.write('%g %g %g\n' % x)
    return psp                  # write and return pattern counters

#-----------------------------------------------------------------------

if __name__ == '__main__':
    grids = [ [ ( 'ncnt',   [   100 ] ),
                ( 'period', [     3 ] ),
                ( 'rate',   [    20 ] ),
                ( 'target', [   'c' ] ),
                ( 'width',  [ 0.012, 0.009, 0.008, 0.006,
                              0.005, 0.004, 0.003, 0.002 ] ),
                ( 'runs',   [ 10000 ] ) ] ]

    run(counts, grids, int(argv[1])-1 if len(argv) > 1 else -1)
